import std.algorithm, std.regex, std.range, std.stdio, std.string, std.ascii;

struct PersonaData
{
    ulong totalWordsSpoken;
    ulong[string] wordCount;
}

void addParagraph(string line, ref PersonaData[string] info)
{
    line = strip(line);
    auto sentence = line.find(". ");
    if (sentence.empty)
    {
        return;
    }
    auto persona = line[0 .. $ - sentence.length];
    sentence = toLower(strip(sentence[2 .. $]));
    auto words = split(sentence, regex("[ \t,.;:?]+"));
    auto data = persona in info;
    if (data)
    {
        data.totalWordsSpoken += words.length;
        foreach (word; words)
            ++data.wordCount[word];
    }
    else
    {
        PersonaData newData;
        newData.totalWordsSpoken = words.length;
        foreach (word; words)
            newData.wordCount[word] = 1;
        info[persona] = newData;
    }
}

void printResults(PersonaData[string] info)
{
    foreach (persona, data; info)
    {
        writefln("%20s %6u %6u", persona, data.totalWordsSpoken,
                data.wordCount.length);
    }
}

void main()
{
    PersonaData[string] info;
    string currentParagraph;
    auto f = File("./source/hamlet.txt", "r");
    foreach (line; f.byLine)
    {
        if (line.startsWith("    ")
                && line.length > 4
                && isAlpha(line[4]))
        {
            currentParagraph ~= line[3 .. $];
        }
        else if (line.startsWith("  ")
                && line.length > 2
                && isAlpha(line[2]))
        {
            addParagraph(currentParagraph, info);
            currentParagraph = line[2 .. $].idup;
        }
    }
    printResults(info);
}
